@@ -35,6 +35,9 @@ namespace trng {
       static T sqrt_2pi() throw () {
 	return T(0);
       }
+      static T one_over_pi() throw () {
+	return T(0);
+      }
       static T one_over_sqrt_2() throw () {
 	return T(0);
       }
@@ -52,6 +55,7 @@ namespace trng {
       TRNG_NEW_CONSTANT(float, .693147180559945309417232f, ln_2);
       TRNG_NEW_CONSTANT(float, 1.41421356237309504880169f, sqrt_2);
       TRNG_NEW_CONSTANT(float, 2.50662827463100050241577f, sqrt_2pi);
+      TRNG_NEW_CONSTANT(float, 0.3183098861837906715377676f, one_over_pi);
       TRNG_NEW_CONSTANT(float, .707106781186547524400845f, one_over_sqrt_2);
       TRNG_NEW_CONSTANT(float, .398942280401432677939946f, one_over_sqrt_2pi)
     };
@@ -65,6 +69,7 @@ namespace trng {
       TRNG_NEW_CONSTANT(double, .693147180559945309417232, ln_2);
       TRNG_NEW_CONSTANT(double, 1.41421356237309504880169, sqrt_2);
       TRNG_NEW_CONSTANT(double, 2.50662827463100050241577, sqrt_2pi);
+      TRNG_NEW_CONSTANT(double, 0.3183098861837906715377676, one_over_pi);
       TRNG_NEW_CONSTANT(double, .707106781186547524400845, one_over_sqrt_2);
       TRNG_NEW_CONSTANT(double, .398942280401432677939946, one_over_sqrt_2pi)
     };
@@ -79,6 +84,7 @@ namespace trng {
       TRNG_NEW_CONSTANT(long double, 1.41421356237309504880169l, sqrt_2);
       TRNG_NEW_CONSTANT(long double, 2.50662827463100050241577l, sqrt_2pi);
       TRNG_NEW_CONSTANT(long double, .707106781186547524400845l, one_over_sqrt_2);
+      TRNG_NEW_CONSTANT(long double, 0.3183098861837906715377676l, one_over_pi);
       TRNG_NEW_CONSTANT(long double, .398942280401432677939946l, one_over_sqrt_2pi)
     };
     
