@@ -46,7 +46,7 @@ namespace trng {
       return S.r[S.index];
     }
     static const result_type min=0;
-    static const result_type max=min-1;  // an ugly hack
+    static const result_type max=~result_type(0);
     
     // Parameter and status classes
     class status_type;
@@ -73,7 +73,7 @@ namespace trng {
 	return true;
       }
       friend bool operator!=(const status_type &a, const status_type &b) {
-	return !(a==b);
+	return not (a==b);
       }
 
       // Streamable concept
@@ -157,7 +157,7 @@ namespace trng {
     }
       
     friend bool operator!=(const lagfib2plus &R1, const lagfib2plus &R2) {
-      return !(R1==R2);
+      return not (R1==R2);
     }
     
     // Streamable concept
@@ -198,7 +198,7 @@ namespace trng {
 		 << A << '_' << B;
 	int i=0;
 	const char *p=name_str.str().c_str();
-	while (p[i]!='\0' && i<63) {
+	while (p[i]!='\0' and i<63) {
 	  name_c_str[i]=p[i];
 	  ++i;
 	}
@@ -207,7 +207,7 @@ namespace trng {
       return name_c_str;
     }
     long operator()(long x) const {
-      return static_cast<long>(utility::uniformco(*this)*x);
+      return static_cast<long>(utility::uniformco<double, lagfib2plus>(*this)*x);
     }
     
   private:
