@@ -30,6 +30,7 @@
 namespace trng {
 
   // uniform random number generator class
+  template<typename float_t=double>
   class rayleigh_dist {
   public:
     typedef double result_type;
@@ -37,13 +38,49 @@ namespace trng {
     
     class param_type {
     private:
-      double nu_;
+      result_type nu_;
     public:
-      double nu() const { return nu_; }
-      void nu(double nu_new) { nu_=nu_new; }
-      explicit param_type(double nu) : nu_(nu) {
+      result_type nu() const { return nu_; }
+      void nu(result_type nu_new) { nu_=nu_new; }
+      param_type() : nu_(1) {
       }
+      explicit param_type(result_type nu) : nu_(nu) {
+      }
+
       friend class rayleigh_dist;
+
+      // Streamable concept
+      template<typename char_t, typename traits_t>
+      friend std::basic_ostream<char_t, traits_t> &
+      operator<<(std::basic_ostream<char_t, traits_t> &out,
+		 const param_type &p) {
+	std::ios_base::fmtflags flags(out.flags());
+	out.flags(std::ios_base::dec | std::ios_base::fixed |
+		  std::ios_base::left);
+	out << '('
+	    << std::setprecision(math::numeric_limits<float_t>::digits10+1) 
+	    << p.nu()
+	    << ')';
+	out.flags(flags);
+	return out;
+      }
+      
+      template<typename char_t, typename traits_t>
+      friend std::basic_istream<char_t, traits_t> &
+      operator>>(std::basic_istream<char_t, traits_t> &in,
+		 param_type &p) {
+	float_t nu;
+	std::ios_base::fmtflags flags(in.flags());
+	in.flags(std::ios_base::dec | std::ios_base::fixed |
+		 std::ios_base::left);
+	in >> utility::delim('(')
+	   >> nu >> utility::delim(')');
+	if (in)
+	  p=rayleigh_dist::param_type(nu);
+	in.flags(flags);
+	return in;
+      }
+
     };
     
   private:
@@ -51,7 +88,7 @@ namespace trng {
    
   public:
     // constructor
-    explicit rayleigh_dist(double nu) : p(nu) {
+    explicit rayleigh_dist(result_type nu) : p(nu) {
     }
     explicit rayleigh_dist(const param_type &p) : p(p) {
     }
@@ -59,100 +96,75 @@ namespace trng {
     void reset() { }
     // random numbers
     template<typename R>
-    double operator()(R &r) {
-      return icdf(utility::uniformoo(r));
+    result_type operator()(R &r) {
+      return icdf(utility::uniformoo<result_type>(r));
     }
     template<typename R>
-    double operator()(R &r, const param_type &p) {
+    result_type operator()(R &r, const param_type &p) {
       rayleigh_dist g(p);
       return g(r);
     }
     // property methods
-    double min() const { return 0.0; }
-    double max() const { return math::numeric_limits<double>::infinity(); }
+    result_type min() const { return 0; }
+    result_type max() const { return math::numeric_limits<result_type>::infinity(); }
     param_type param() const { return p; }
     void param(const param_type &p_new) { p=p_new; }
-    double nu() const { return p.nu(); }
-    void nu(double nu_new) { p.nu(nu_new); }
+    result_type nu() const { return p.nu(); }
+    void nu(result_type nu_new) { p.nu(nu_new); }
     // probability density function  
-    double pdf(double x) const {
-      if (x<=0.0)
-	return 0.0;
-      double t=x/(p.nu()*p.nu());
-      return t*math::exp(-0.5*t*x);
+    result_type pdf(result_type x) const {
+      if (x<=0)
+	return 0;
+      result_type t=x/(p.nu()*p.nu());
+      return t*math::exp(-t*x/2);
     }
     // cumulative density function 
-    double cdf(double x) const {
-      if (x<=0.0)
-	return 0.0;
-      return 1.0-math::exp(-0.5*x*x/(p.nu()*p.nu()));
+    result_type cdf(result_type x) const {
+      if (x<=0)
+	return 0;
+      return 1-math::exp(-x*x/(2*p.nu()*p.nu()));
     }
     // inverse cumulative density function 
-    double icdf(double x) const {
-      return p.nu()*math::sqrt(-2.0*math::ln(1.0-x));
+    result_type icdf(result_type x) const {
+      return p.nu()*math::sqrt(-2*math::ln(1-x));
     }
   };
     
   // -------------------------------------------------------------------
 
   // EqualityComparable concept
-  inline bool operator==(const rayleigh_dist::param_type &p1, 
-			 const rayleigh_dist::param_type &p2) {
+  template<typename float_t>
+  inline bool operator==(const typename rayleigh_dist<float_t>::param_type &p1, 
+			 const typename rayleigh_dist<float_t>::param_type &p2) {
     return p1.nu()==p2.nu();
   }
-  inline bool operator!=(const rayleigh_dist::param_type &p1, 
-			 const rayleigh_dist::param_type &p2) {
-    return !(p1==p2);
+
+  template<typename float_t>
+  inline bool operator!=(const typename rayleigh_dist<float_t>::param_type &p1, 
+			 const typename rayleigh_dist<float_t>::param_type &p2) {
+    return not (p1==p2);
   }
-  
-  // Streamable concept
-  template<typename char_t, typename traits_t>
-  std::basic_ostream<char_t, traits_t> &
-  operator<<(std::basic_ostream<char_t, traits_t> &out,
-	     const rayleigh_dist::param_type &p) {
-    std::ios_base::fmtflags flags(out.flags());
-    out.flags(std::ios_base::dec | std::ios_base::fixed |
-	      std::ios_base::left);
-    out << '('
-	<< std::setprecision(17) << p.nu()
-	<< ')';
-    out.flags(flags);
-    return out;
-  }
-  
-  template<typename char_t, typename traits_t>
-  std::basic_istream<char_t, traits_t> &
-  operator>>(std::basic_istream<char_t, traits_t> &in,
-	     rayleigh_dist::param_type &p) {
-    double nu;
-    std::ios_base::fmtflags flags(in.flags());
-    in.flags(std::ios_base::dec | std::ios_base::fixed |
-	     std::ios_base::left);
-    in >> utility::delim('(')
-       >> nu >> utility::delim(')');
-    if (in)
-      p=rayleigh_dist::param_type(nu);
-    in.flags(flags);
-    return in;
-  }
-  
+    
   // -------------------------------------------------------------------
 
   // EqualityComparable concept
-  inline bool operator==(const rayleigh_dist &g1, 
-			 const rayleigh_dist &g2) {
+  template<typename float_t>
+  inline bool operator==(const rayleigh_dist<float_t> &g1, 
+			 const rayleigh_dist<float_t> &g2) {
     return g1.param()==g2.param();
   }
-  inline bool operator!=(const rayleigh_dist &g1, 
-			 const rayleigh_dist &g2) {
+
+  template<typename float_t>
+  inline bool operator!=(const rayleigh_dist<float_t> &g1, 
+			 const rayleigh_dist<float_t> &g2) {
     return g1.param()!=g2.param();
   }
   
   // Streamable concept
-  template<typename char_t, typename traits_t>
+  template<typename char_t, typename traits_t, typename float_t>
   std::basic_ostream<char_t, traits_t> &
   operator<<(std::basic_ostream<char_t, traits_t> &out,
-	     const rayleigh_dist &g) {
+	     const rayleigh_dist<float_t> &g) {
     std::ios_base::fmtflags flags(out.flags());
     out.flags(std::ios_base::dec | std::ios_base::fixed |
 	      std::ios_base::left);
@@ -161,11 +173,11 @@ namespace trng {
     return out;
   }
   
-  template<typename char_t, typename traits_t>
+  template<typename char_t, typename traits_t, typename float_t>
   std::basic_istream<char_t, traits_t> &
   operator>>(std::basic_istream<char_t, traits_t> &in,
-	     rayleigh_dist &g) {
-    rayleigh_dist::param_type p;
+	     rayleigh_dist<float_t> &g) {
+    typename rayleigh_dist<float_t>::param_type p;
     std::ios_base::fmtflags flags(in.flags());
     in.flags(std::ios_base::dec | std::ios_base::fixed |
 	     std::ios_base::left);
