@@ -26,31 +26,29 @@ namespace trng {
   // EqualityComparable concept
   bool operator==(const yarn2::parameter_type &P1,
                   const yarn2::parameter_type &P2) {
-    return P1.a1==P2.a1 && P1.a2==P2.a2 && P1.g==P2.g;
+    return P1.a1==P2.a1 and P1.a2==P2.a2;
   }
 
   bool operator!=(const yarn2::parameter_type &P1,
                   const yarn2::parameter_type &P2) {
-    return !(P1==P2);
+    return not (P1==P2);
   }
 
   // Equality comparable concept
   bool operator==(const yarn2::status_type &S1,
                   const yarn2::status_type &S2) {
-    return S1.r1==S2.r1 && S1.r2==S2.r2;
+    return S1.r1==S2.r1 and S1.r2==S2.r2;
   }
 
   bool operator!=(const yarn2::status_type &S1,
                   const yarn2::status_type &S2) {
-    return !(S1==S2);
+    return not (S1==S2);
   }
 
   const yarn2::parameter_type 
-  yarn2::LEcuyer1=parameter_type(1498809829l, 1160990996l, 
-				 123567893l);
+  yarn2::LEcuyer1=parameter_type(1498809829l, 1160990996l);
   const yarn2::parameter_type 
-  yarn2::LEcuyer2=parameter_type(46325l, 1084587l,
-				 123567893l);
+  yarn2::LEcuyer2=parameter_type(46325l, 1084587l);
   
   // Random number engine concept
   yarn2::yarn2(yarn2::parameter_type P) :
@@ -85,16 +83,16 @@ namespace trng {
 
   // Equality comparable concept
   bool operator==(const yarn2 &R1, const yarn2 &R2) {
-    return R1.P==R2.P && R1.S==R2.S;
+    return R1.P==R2.P and R1.S==R2.S;
   }
 
   bool operator!=(const yarn2 &R1, const yarn2 &R2) {
-    return !(R1==R2);
+    return not (R1==R2);
   }
 
   // Parallel random number generator concept
   void yarn2::split(unsigned int s, unsigned int n) {
-    if (s<1 || n>=s)
+    if (s<1 or n>=s)
       throw std::invalid_argument("invalid argument for trng::yarn2::split");
     long q0, q1, q2, q3;
     if (s>1) {
@@ -173,5 +171,7 @@ namespace trng {
       t=0l;
     S.r1=S.r2;  S.r2=t;
   }
-  
+
+  utility::power<yarn2::modulus, yarn2::gen> yarn2::parameter_type::g;
+
 }
