@@ -26,31 +26,29 @@ namespace trng {
   // EqualityComparable concept
   bool operator==(const yarn3s::parameter_type &P1,
                   const yarn3s::parameter_type &P2) {
-    return P1.a1==P2.a1 && P1.a2==P2.a2 && P1.a3==P2.a3 && P1.g==P2.g;
+    return P1.a1==P2.a1 and P1.a2==P2.a2 and P1.a3==P2.a3;
   }
 
   bool operator!=(const yarn3s::parameter_type &P1,
                   const yarn3s::parameter_type &P2) {
-    return !(P1==P2);
+    return not (P1==P2);
   }
 
   // Equality comparable concept
   bool operator==(const yarn3s::status_type &S1,
                   const yarn3s::status_type &S2) {
-    return S1.r1==S2.r1 && S1.r2==S2.r2 && S1.r3==S2.r3;
+    return S1.r1==S2.r1 and S1.r2==S2.r2 and S1.r3==S2.r3;
   }
 
   bool operator!=(const yarn3s::status_type &S1,
                   const yarn3s::status_type &S2) {
-    return !(S1==S2);
+    return not (S1==S2);
   }
 
   const yarn3s::parameter_type
-  yarn3s::trng0=parameter_type(2025213985l, 1112953677l, 2038969601l,
-			       1616076847l);
+  yarn3s::trng0=parameter_type(2025213985l, 1112953677l, 2038969601l);
   const yarn3s::parameter_type
-  yarn3s::trng1=parameter_type(1287767370l, 1045931779l, 58150106l,
-			       1616076847l);
+  yarn3s::trng1=parameter_type(1287767370l, 1045931779l, 58150106l);
   
   // Random number engine concept
   yarn3s::yarn3s(yarn3s::parameter_type P) :
@@ -90,16 +88,16 @@ namespace trng {
 
   // Equality comparable concept
   bool operator==(const yarn3s &R1, const yarn3s &R2) {
-    return R1.P==R2.P && R1.S==R2.S;
+    return R1.P==R2.P and R1.S==R2.S;
   }
 
   bool operator!=(const yarn3s &R1, const yarn3s &R2) {
-    return !(R1==R2);
+    return not (R1==R2);
   }
 
   // Parallel random number generator concept
   void yarn3s::split(unsigned int s, unsigned int n) {
-    if (s<1 || n>=s)
+    if (s<1 or n>=s)
       throw std::invalid_argument("invalid argument for trng::yarn3s::split");
     long q0, q1, q2, q3, q4, q5;
     if (s>1) {
@@ -198,5 +196,7 @@ namespace trng {
       t=0l;
     S.r1=S.r2;  S.r2=S.r3;  S.r3=t;
   }
-  
+
+  utility::power<yarn3s::modulus, yarn3s::gen> yarn3s::parameter_type::g;
+
 }
