@@ -36,9 +36,10 @@ namespace trng {
     result_type operator()() const;
   private:
     static const result_type modulus=2147483647l;
+    static const result_type gen=123567893l;
   public:
     static const result_type min=0l;
-    static const result_type max=2147483646l;
+    static const result_type max=modulus-1;
 
     // Parameter and status classes
     class parameter_type;
@@ -46,15 +47,14 @@ namespace trng {
 
     class parameter_type {
       result_type a1, a2, a3, a4, a5;
-      utility::power<2147483647l> g;
+      static utility::power<yarn5::modulus, yarn5::gen> g;
     public:
       parameter_type() :
-        a1(0), a2(0), a3(0), a4(0), a5(0), g() { };
+        a1(0), a2(0), a3(0), a4(0), a5(0) { };
       parameter_type(result_type a1, result_type a2,
                      result_type a3, result_type a4,
-		     result_type a5,
-                     result_type g) :
-        a1(a1), a2(a2), a3(a3), a4(a4), a5(a5), g(g) { };
+		     result_type a5) :
+        a1(a1), a2(a2), a3(a3), a4(a4), a5(a5) { };
 
       friend class yarn5;
 
@@ -71,7 +71,7 @@ namespace trng {
         out.flags(std::ios_base::dec | std::ios_base::fixed |
                   std::ios_base::left);
         out << '('
-            << P.a1 << ' ' << P.a2 << ' ' << P.a3 << ' ' << P.a4 << ' ' << P.a5 << ' ' << P.g
+            << P.a1 << ' ' << P.a2 << ' ' << P.a3 << ' ' << P.a4 << ' ' << P.a5
             << ')';
         out.flags(flags);
         return out;
@@ -90,8 +90,7 @@ namespace trng {
            >> P_new.a2 >> utility::delim(' ')
            >> P_new.a3 >> utility::delim(' ')
            >> P_new.a4 >> utility::delim(' ')
-           >> P_new.a5 >> utility::delim(' ')
-           >> P_new.g  >> utility::delim(')');
+           >> P_new.a5 >> utility::delim(')');
         if (in)
           P=P_new;
         in.flags(flags);
@@ -250,18 +249,12 @@ namespace trng {
 			 static_cast<unsigned long long>(S.r3)+
 			 static_cast<unsigned long long>(P.a4)*
 			 static_cast<unsigned long long>(S.r4));
-    if (t>=2ull*2147483647ull*2147483647ull)
-      t-=2ull*2147483647ull*2147483647ull;
+    if (t>=2ull*modulus*modulus)
+      t-=2ull*modulus*modulus;
     t+=static_cast<unsigned long long>(P.a5)*
       static_cast<unsigned long long>(S.r5);
-    t=(t&0x7fffffffull)+(t>>31);
-    if (t>=4ull*2147483647ull)
-      t-=4ull*2147483647ull;
-    if (t>=2ull*2147483647ull)
-      t-=2ull*2147483647ull;
-    if (t>=2147483647ull)
-      t-=2147483647ull;
-    S.r5=S.r4;  S.r4=S.r3;  S.r3=S.r2;  S.r2=S.r1;  S.r1=t;
+    S.r5=S.r4;  S.r4=S.r3;  S.r3=S.r2;  S.r2=S.r1;  S.r1=utility::modulo<modulus, 5>(t);
+
   }
 
   inline yarn5::result_type yarn5::operator()() const {
@@ -270,7 +263,7 @@ namespace trng {
   }
   
   inline long yarn5::operator()(long x) const {
-    return static_cast<long>(utility::uniformco(*this)*x);
+    return static_cast<long>(utility::uniformco<double, yarn5>(*this)*x);
   }
 
 }
